package com.sun.org.apache.xml.internal.resolver;

import sun.reflect.misc.ReflectUtil;
import java.net.MalformedURLException;
import java.util.StringTokenizer;
import java.util.Vector;
import java.io.InputStream;
import java.io.IOException;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import com.sun.org.apache.xerces.internal.utils.SecuritySupport;
import com.sun.org.apache.xml.internal.resolver.helpers.Debug;
import java.net.URL;
import java.util.ResourceBundle;
import com.sun.org.apache.xml.internal.resolver.helpers.BootstrapResolver;

public class CatalogManager
{
    private static final String pFiles = "xml.catalog.files";
    private static final String pVerbosity = "xml.catalog.verbosity";
    private static final String pPrefer = "xml.catalog.prefer";
    private static final String pStatic = "xml.catalog.staticCatalog";
    private static final String pAllowPI = "xml.catalog.allowPI";
    private static final String pClassname = "xml.catalog.className";
    private static final String pIgnoreMissing = "xml.catalog.ignoreMissing";
    private static final CatalogManager staticManager;
    private BootstrapResolver bResolver;
    private boolean ignoreMissingProperties;
    private ResourceBundle resources;
    private String propertyFile;
    private URL propertyFileURI;
    private String defaultCatalogFiles;
    private String catalogFiles;
    private boolean fromPropertiesFile;
    private int defaultVerbosity;
    private Integer verbosity;
    private boolean defaultPreferPublic;
    private Boolean preferPublic;
    private boolean defaultUseStaticCatalog;
    private Boolean useStaticCatalog;
    private static volatile Catalog staticCatalog;
    private boolean defaultOasisXMLCatalogPI;
    private Boolean oasisXMLCatalogPI;
    private boolean defaultRelativeCatalogs;
    private Boolean relativeCatalogs;
    private String catalogClassName;
    private boolean overrideDefaultParser;
    public Debug debug;
    
    public CatalogManager() {
        this.bResolver = new BootstrapResolver();
        this.ignoreMissingProperties = (SecuritySupport.getSystemProperty("xml.catalog.ignoreMissing") != null || SecuritySupport.getSystemProperty("xml.catalog.files") != null);
        this.propertyFile = "CatalogManager.properties";
        this.propertyFileURI = null;
        this.defaultCatalogFiles = "./xcatalog";
        this.catalogFiles = null;
        this.fromPropertiesFile = false;
        this.defaultVerbosity = 1;
        this.verbosity = null;
        this.defaultPreferPublic = true;
        this.preferPublic = null;
        this.defaultUseStaticCatalog = true;
        this.useStaticCatalog = null;
        this.defaultOasisXMLCatalogPI = true;
        this.oasisXMLCatalogPI = null;
        this.defaultRelativeCatalogs = true;
        this.relativeCatalogs = null;
        this.catalogClassName = null;
        this.debug = null;
        this.init();
    }
    
    public CatalogManager(String propertyFile) {
        this.bResolver = new BootstrapResolver();
        this.ignoreMissingProperties = (SecuritySupport.getSystemProperty("xml.catalog.ignoreMissing") != null || SecuritySupport.getSystemProperty("xml.catalog.files") != null);
        this.propertyFile = "CatalogManager.properties";
        this.propertyFileURI = null;
        this.defaultCatalogFiles = "./xcatalog";
        this.catalogFiles = null;
        this.fromPropertiesFile = false;
        this.defaultVerbosity = 1;
        this.verbosity = null;
        this.defaultPreferPublic = true;
        this.preferPublic = null;
        this.defaultUseStaticCatalog = true;
        this.useStaticCatalog = null;
        this.defaultOasisXMLCatalogPI = true;
        this.oasisXMLCatalogPI = null;
        this.defaultRelativeCatalogs = true;
        this.relativeCatalogs = null;
        this.catalogClassName = null;
        this.debug = null;
        this.propertyFile = propertyFile;
        this.init();
    }
    
    private void init() {
        this.debug = new Debug();
        if (System.getSecurityManager() == null) {
            this.overrideDefaultParser = true;
        }
        this.queryVerbosityFromSysProp();
    }
    
    public void setBootstrapResolver(BootstrapResolver resolver) {
        this.bResolver = resolver;
    }
    
    public BootstrapResolver getBootstrapResolver() {
        return this.bResolver;
    }
    
    private void queryVerbosityFromSysProp() {
        String verbStr = SecuritySupport.getSystemProperty("xml.catalog.verbosity");
        if (verbStr != null) {
            try {
                int verb = Integer.parseInt(verbStr.trim());
                this.verbosity = new Integer(verb);
                this.debug.setDebug(verb);
            }
            catch (Exception e) {
                System.err.println("Cannot parse verbosity: \"" + verbStr + "\"");
            }
        }
    }
    
    private synchronized void readProperties() {
        try {
            this.propertyFileURI = CatalogManager.class.getResource("/" + this.propertyFile);
            InputStream in = CatalogManager.class.getResourceAsStream("/" + this.propertyFile);
            if (in == null) {
                if (!this.ignoreMissingProperties) {
                    System.err.println("Cannot find " + this.propertyFile);
                    this.ignoreMissingProperties = true;
                }
                return;
            }
            else {
                this.resources = new PropertyResourceBundle(in);
            }
        }
        catch (MissingResourceException mre) {
            if (!this.ignoreMissingProperties) {
                System.err.println("Cannot read " + this.propertyFile);
            }
        }
        catch (IOException e) {
            if (!this.ignoreMissingProperties) {
                System.err.println("Failure trying to read " + this.propertyFile);
            }
        }
        if (this.verbosity == null) {
            try {
                String verbStr = this.resources.getString("verbosity");
                int verb = Integer.parseInt(verbStr.trim());
                this.debug.setDebug(verb);
                this.verbosity = new Integer(verb);
            }
            catch (Exception ex) {}
        }
    }
    
    public static CatalogManager getStaticManager() {
        return CatalogManager.staticManager;
    }
    
    public boolean getIgnoreMissingProperties() {
        return this.ignoreMissingProperties;
    }
    
    public void setIgnoreMissingProperties(boolean ignore) {
        this.ignoreMissingProperties = ignore;
    }
    
    @Deprecated
    public void ignoreMissingProperties(boolean ignore) {
        this.setIgnoreMissingProperties(ignore);
    }
    
    private int queryVerbosity() {
        String defaultVerbStr = Integer.toString(this.defaultVerbosity);
        String verbStr = SecuritySupport.getSystemProperty("xml.catalog.verbosity");
        if (verbStr == null) {
            if (this.resources == null) {
                this.readProperties();
            }
            if (this.resources != null) {
                try {
                    verbStr = this.resources.getString("verbosity");
                }
                catch (MissingResourceException e) {
                    verbStr = defaultVerbStr;
                }
            }
            else {
                verbStr = defaultVerbStr;
            }
        }
        int verb = this.defaultVerbosity;
        try {
            verb = Integer.parseInt(verbStr.trim());
        }
        catch (Exception e2) {
            System.err.println("Cannot parse verbosity: \"" + verbStr + "\"");
        }
        if (this.verbosity == null) {
            this.debug.setDebug(verb);
            this.verbosity = new Integer(verb);
        }
        return verb;
    }
    
    public int getVerbosity() {
        if (this.verbosity == null) {
            this.verbosity = new Integer(this.queryVerbosity());
        }
        return (int)this.verbosity;
    }
    
    public void setVerbosity(int verbosity) {
        this.verbosity = new Integer(verbosity);
        this.debug.setDebug(verbosity);
    }
    
    @Deprecated
    public int verbosity() {
        return this.getVerbosity();
    }
    
    private boolean queryRelativeCatalogs() {
        if (this.resources == null) {
            this.readProperties();
        }
        if (this.resources == null) {
            return this.defaultRelativeCatalogs;
        }
        try {
            String allow = this.resources.getString("relative-catalogs");
            return allow.equalsIgnoreCase("true") || allow.equalsIgnoreCase("yes") || allow.equalsIgnoreCase("1");
        }
        catch (MissingResourceException e) {
            return this.defaultRelativeCatalogs;
        }
    }
    
    public boolean getRelativeCatalogs() {
        if (this.relativeCatalogs == null) {
            this.relativeCatalogs = (this.queryRelativeCatalogs() ? Boolean.TRUE : Boolean.FALSE);
        }
        return (boolean)this.relativeCatalogs;
    }
    
    public void setRelativeCatalogs(boolean relative) {
        this.relativeCatalogs = (relative ? Boolean.TRUE : Boolean.FALSE);
    }
    
    @Deprecated
    public boolean relativeCatalogs() {
        return this.getRelativeCatalogs();
    }
    
    private String queryCatalogFiles() {
        String catalogList = SecuritySupport.getSystemProperty("xml.catalog.files");
        this.fromPropertiesFile = false;
        if (catalogList == null) {
            if (this.resources == null) {
                this.readProperties();
            }
            if (this.resources != null) {
                try {
                    catalogList = this.resources.getString("catalogs");
                    this.fromPropertiesFile = true;
                }
                catch (MissingResourceException e) {
                    System.err.println(this.propertyFile + ": catalogs not found.");
                    catalogList = null;
                }
            }
        }
        if (catalogList == null) {
            catalogList = this.defaultCatalogFiles;
            return catalogList;
        }
        return catalogList;
    }
    
    public Vector getCatalogFiles() {
        if (this.catalogFiles == null) {
            this.catalogFiles = this.queryCatalogFiles();
        }
        StringTokenizer files = new StringTokenizer(this.catalogFiles, ";");
        Vector<String> catalogs = new Vector();
        while (files.hasMoreTokens()) {
            String catalogFile = files.nextToken();
            URL absURI = null;
            if (this.fromPropertiesFile && !this.relativeCatalogs()) {
                try {
                    absURI = new URL(this.propertyFileURI, catalogFile);
                    catalogFile = absURI.toString();
                }
                catch (MalformedURLException mue) {
                    absURI = null;
                }
            }
            catalogs.add(catalogFile);
        }
        return catalogs;
    }
    
    public void setCatalogFiles(String fileList) {
        this.catalogFiles = fileList;
        this.fromPropertiesFile = false;
    }
    
    @Deprecated
    public Vector catalogFiles() {
        return this.getCatalogFiles();
    }
    
    private boolean queryPreferPublic() {
        String prefer = SecuritySupport.getSystemProperty("xml.catalog.prefer");
        if (prefer == null) {
            if (this.resources == null) {
                this.readProperties();
            }
            if (this.resources == null) {
                return this.defaultPreferPublic;
            }
            try {
                prefer = this.resources.getString("prefer");
            }
            catch (MissingResourceException e) {
                return this.defaultPreferPublic;
            }
        }
        if (prefer == null) {
            return this.defaultPreferPublic;
        }
        return prefer.equalsIgnoreCase("public");
    }
    
    public boolean getPreferPublic() {
        if (this.preferPublic == null) {
            this.preferPublic = (this.queryPreferPublic() ? Boolean.TRUE : Boolean.FALSE);
        }
        return (boolean)this.preferPublic;
    }
    
    public void setPreferPublic(boolean preferPublic) {
        this.preferPublic = (preferPublic ? Boolean.TRUE : Boolean.FALSE);
    }
    
    @Deprecated
    public boolean preferPublic() {
        return this.getPreferPublic();
    }
    
    private boolean queryUseStaticCatalog() {
        String staticCatalog = SecuritySupport.getSystemProperty("xml.catalog.staticCatalog");
        if (staticCatalog == null) {
            if (this.resources == null) {
                this.readProperties();
            }
            if (this.resources == null) {
                return this.defaultUseStaticCatalog;
            }
            try {
                staticCatalog = this.resources.getString("static-catalog");
            }
            catch (MissingResourceException e) {
                return this.defaultUseStaticCatalog;
            }
        }
        if (staticCatalog == null) {
            return this.defaultUseStaticCatalog;
        }
        return staticCatalog.equalsIgnoreCase("true") || staticCatalog.equalsIgnoreCase("yes") || staticCatalog.equalsIgnoreCase("1");
    }
    
    public boolean getUseStaticCatalog() {
        if (this.useStaticCatalog == null) {
            this.useStaticCatalog = (this.queryUseStaticCatalog() ? Boolean.TRUE : Boolean.FALSE);
        }
        return (boolean)this.useStaticCatalog;
    }
    
    public void setUseStaticCatalog(boolean useStatic) {
        this.useStaticCatalog = (useStatic ? Boolean.TRUE : Boolean.FALSE);
    }
    
    @Deprecated
    public boolean staticCatalog() {
        return this.getUseStaticCatalog();
    }
    
    public Catalog getPrivateCatalog() {
        Catalog catalog = CatalogManager.staticCatalog;
        if (this.useStaticCatalog == null) {
            this.useStaticCatalog = (this.getUseStaticCatalog() ? Boolean.TRUE : Boolean.FALSE);
        }
        if (catalog != null && (boolean)this.useStaticCatalog) {
            return catalog;
        }
        try {
            String catalogClassName = this.getCatalogClassName();
            if (catalogClassName == null) {
                catalog = new Catalog();
            }
            else {
                try {
                    catalog = (Catalog)ReflectUtil.forName(catalogClassName).newInstance();
                }
                catch (ClassNotFoundException cnfe) {
                    this.debug.message(1, "Catalog class named '" + catalogClassName + "' could not be found. Using default.");
                    catalog = new Catalog();
                }
                catch (ClassCastException cnfe2) {
                    this.debug.message(1, "Class named '" + catalogClassName + "' is not a Catalog. Using default.");
                    catalog = new Catalog();
                }
            }
            catalog.setCatalogManager(this);
            catalog.setupReaders();
            catalog.loadSystemCatalogs();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        if ((boolean)this.useStaticCatalog) {
            CatalogManager.staticCatalog = catalog;
        }
        return catalog;
    }
    
    public Catalog getCatalog() {
        Catalog catalog = CatalogManager.staticCatalog;
        if (this.useStaticCatalog == null) {
            this.useStaticCatalog = (this.getUseStaticCatalog() ? Boolean.TRUE : Boolean.FALSE);
        }
        if (catalog == null || !(boolean)this.useStaticCatalog) {
            catalog = this.getPrivateCatalog();
            if ((boolean)this.useStaticCatalog) {
                CatalogManager.staticCatalog = catalog;
            }
        }
        return catalog;
    }
    
    public boolean queryAllowOasisXMLCatalogPI() {
        String allow = SecuritySupport.getSystemProperty("xml.catalog.allowPI");
        if (allow == null) {
            if (this.resources == null) {
                this.readProperties();
            }
            if (this.resources == null) {
                return this.defaultOasisXMLCatalogPI;
            }
            try {
                allow = this.resources.getString("allow-oasis-xml-catalog-pi");
            }
            catch (MissingResourceException e) {
                return this.defaultOasisXMLCatalogPI;
            }
        }
        if (allow == null) {
            return this.defaultOasisXMLCatalogPI;
        }
        return allow.equalsIgnoreCase("true") || allow.equalsIgnoreCase("yes") || allow.equalsIgnoreCase("1");
    }
    
    public boolean getAllowOasisXMLCatalogPI() {
        if (this.oasisXMLCatalogPI == null) {
            this.oasisXMLCatalogPI = (this.queryAllowOasisXMLCatalogPI() ? Boolean.TRUE : Boolean.FALSE);
        }
        return (boolean)this.oasisXMLCatalogPI;
    }
    
    public boolean overrideDefaultParser() {
        return this.overrideDefaultParser;
    }
    
    public void setAllowOasisXMLCatalogPI(boolean allowPI) {
        this.oasisXMLCatalogPI = (allowPI ? Boolean.TRUE : Boolean.FALSE);
    }
    
    @Deprecated
    public boolean allowOasisXMLCatalogPI() {
        return this.getAllowOasisXMLCatalogPI();
    }
    
    public String queryCatalogClassName() {
        String className = SecuritySupport.getSystemProperty("xml.catalog.className");
        if (className == null) {
            if (this.resources == null) {
                this.readProperties();
            }
            if (this.resources == null) {
                return null;
            }
            try {
                return this.resources.getString("catalog-class-name");
            }
            catch (MissingResourceException e) {
                return null;
            }
        }
        else {
            return className;
        }
    }
    
    public String getCatalogClassName() {
        if (this.catalogClassName == null) {
            this.catalogClassName = this.queryCatalogClassName();
        }
        return this.catalogClassName;
    }
    
    public void setCatalogClassName(String className) {
        this.catalogClassName = className;
    }
    
    @Deprecated
    public String catalogClassName() {
        return this.getCatalogClassName();
    }
    
    static {
        staticManager = new CatalogManager();
        CatalogManager.staticCatalog = null;
    }
}
